#include <math.h>

void engine_3d_find_rotation(struct ENGINE_VECTOR_3D * rot,float x, float y, float z) {
	float distance = sqrtf(x * x + y * y);
	rot->x = atan2f(y,x) * (180.0/M_PI);
	rot->y = atan2f(z,distance) * (180.0/M_PI);
	rot->z = 0.0;
}

void engine_3d_rotate_position(struct ENGINE_VECTOR_3D * pos,float x, float y, float z, float rx, float ry, float rz) {
	rx = rx * (M_PI/180.0);
	ry = ry * (M_PI/180.0);
	rz = rz * (M_PI/180.0);
	float sinX = sinf(rx);
	float cosX = cosf(rx);
	float sinY = sinf(ry);
	float cosY = cosf(ry);
	float sinZ = sinf(rz);
	float cosZ = cosf(rz);
	pos->x = x * (cosY * cosZ) + y * (sinX * sinY * cosZ - cosX * sinZ) + z * (cosX * sinY * cosZ + sinX * sinZ);
	pos->y = x * (cosY * sinZ) + y * (sinX * sinY * sinZ + cosX * cosZ) + z * (cosX * sinY * sinZ - sinX * cosZ);
	pos->z = x * (-sinY) + y * (sinX * cosY) + z * (cosX * cosY);
}

void engine_3d_project(struct ENGINE_VECTOR_2D * screen,float x, float y, float z, float rx, float ry, float rz, float fov) {
	struct ENGINE_VECTOR_3D pos;
	pos.x = x;
	pos.y = y;
	pos.z = z;
	
	engine_3d_rotate_position(&pos,pos.x,pos.y,pos.z,0.0,ry,0.0);
	engine_3d_rotate_position(&pos,pos.x,pos.y,pos.z,0.0,0.0,rx);
	pos.x = -pos.x; pos.y = -pos.y; pos.z = -pos.z;
	struct ENGINE_VECTOR_3D rot;
	engine_3d_find_rotation(&rot,pos.x,pos.y,pos.z);
	
	// rot.y (screen X)
	while (rot.y >  180.0) { rot.y -= 360.0; }
	while (rot.y < -180.0) { rot.y += 360.0; }
	rot.y = rot.y / (fov * 0.5);
	
	// rot.x (screen Y)
	while (rot.x >  180.0) { rot.x -= 360.0; }
	while (rot.x < -180.0) { rot.x += 360.0; }
	rot.x = rot.x / (fov * 0.5);
	
	screen->x = rot.y;
	screen->y = rot.x;
}